/*
 * This library is copyright Carter Soft, 2012
 * 
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package au.cartersoft.GeoClasses;
/**
 * A package continaing tools useful for programmers writing map-based applications using the Google Maps API
 * @author      Carter Soft <sales@cartersoft.com.au>
 * @version     1.0                            
 * @since       1.0
 * @         
 */

import com.google.android.maps.GeoPoint;

/**
 * A class containing methods for performing calculations on geospatial information
 */
public class DistanceCalculator {
 
	//earths radius (mean radius = 6,371km)
   private static final double radius = 6371;
	
   /**
	 * Finds the distance between two points
	 * @param  startpt A GeoPoint containing the point to measure from
	 * @param  endpt A Geopoint containing the point to measure to
	 * @return The distance in kilometres between the two points
	 * @see <A HREF="http://code.google.com/android/add-ons/google-apis/reference/com/google/android/maps/GeoPoint.html">com.google.android.maps.GeoPoint</A>
	 */ 
   
   public static double calculateDistance(GeoPoint startpt, GeoPoint endpt) {
 
	  double distance = 0;
 
	  if(startpt != null && endpt != null) {
		  double lat1 = startpt.getLatitudeE6()/1E6;
		  double lat2 = endpt.getLatitudeE6()/1E6;
		  double lon1 = startpt.getLongitudeE6()/1E6;
		  double lon2 = endpt.getLongitudeE6()/1E6;
		  double dLat = Math.toRadians(lat2-lat1);
		  double dLon = Math.toRadians(lon2-lon1);
		  double a = Math.sin(dLat/2) * Math.sin(dLat/2) +
		  Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2)) * 
		  Math.sin(dLon/2) * Math.sin(dLon/2); 
		  double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
		  distance = radius * c;
	  }
 
	  return distance;
   }
   
   /**
	 * Converts the difference between two latitudes as a distance in kilometres
	 * 
	 * Android provides a method to get the latitude span of a map presently displayed on
	 * screen, this method converts that span to kilometres
	 * @param  lat The difference between two latitudes multiplied by 1,000,000 (the exact output of MapView.getLatitudeSpan())
	 * @return The latitude span in kilometres
	 * @see <A HREF="http://code.google.com/android/add-ons/google-apis/reference/com/google/android/maps/MapView.html#getLatitudeSpan()">com.google.maps.MapView#getLatitudeSpan()</A>
	 */
   
   public static double convertLatToDistance(int lat)
   {
  		  double distance = 0;
		  
		  double dLat = lat / 1E6;
		  distance = dLat * 111.12;
	 	  return distance;
	}
   
   /**
	 * Converts the difference between two longitudes as a distance in kilometres
	 * 
	 * Android provides a method to get the longitude span of a map presently displayed on
	 * screen, this method converts that span to kilometres
	 * @param  lat The latitude at which the longitude range is calculated
	 * @param lngDist The difference between two longitudes multiplied by 1,000,000 (the exact output of MapView.getLatitudeSpan())
	 * @return The longitude span in kilometres
	 * @see <A HREF="http://code.google.com/android/add-ons/google-apis/reference/com/google/android/maps/MapView.html#getLatitudeSpan()">com.google.maps.MapView#getLongitudeSpan()</A>
	 */
   
   public static double convertLngToDistance(double lat, int lngDist)
   {
	   double distance = 0;
	   double dLng = lngDist / 1E6;
		   
	   distance = (dLng * 111.12) * Math.cos(lat);
	   if (distance < 0)
	   {
		   distance = distance - distance - distance;
	   }
	   return distance;
   }
}